@@ -966,10 +966,10 @@ static void basic_http_header(request_rec *r, apr_bucket_brigade *bb,
          * Date and Server are less interesting, use TRACE5 for them while
          * using TRACE4 for the other headers.
          */
-        ap_log_rerror(APLOG_MARK, APLOG_TRACE5, 0, r, "  %s: %s", "Date",
+        ap_log_rerror(APLOG_MARK, APLOG_TRACE5, 0, r, "  Date: %s",
                       proxy_date ? proxy_date : date );
         if (server)
-            ap_log_rerror(APLOG_MARK, APLOG_TRACE5, 0, r, "  %s: %s", "Server",
+            ap_log_rerror(APLOG_MARK, APLOG_TRACE5, 0, r, "  Server: %s",
                           server);
     }
 
